<?php

namespace App\Helpers;

use Exception;
use Nette;

/**
 * Helping class for bibtex validation
 *
 */
abstract class Bibtex {

    use Nette\SmartObject;

    protected $pub_type;
    protected $report;
    protected $starmap;
    protected string $pad = '&nbsp;&nbsp;&nbsp;'; //padding string
    protected array $bibtex_fields = array(); //array of fields in bibtex export (e.g.


    /**
     * Factory method. Regarding to the type of publication creates appropriate successor class
     *
     * @param string $pub_type type of publication
     * @return Article|Book|Booklet|Inbook|Incollection|Inproceedings|Manual|Mastersthesis|Misc|Phdthesis|Proceedings|Techreport|Unpublished|null
     */
    public static function create(string $pub_type)
    {
        switch ($pub_type) {
          case 'article':
            return new Article();
          case 'book':
            return new Book();
          case 'booklet':
            return new Booklet();
          case 'inbook':
            return new Inbook();
          case 'incollection':
            return new Incollection();
          case 'inproceedings':
            return new Inproceedings();
          case 'manual':
            return new Manual();
          case 'mastersthesis':
            return new Mastersthesis();
          case 'misc':
            return new Misc();
          case 'phdthesis':
            return new Phdthesis();
          case 'proceedings':
            return new Proceedings();
          case 'techreport':
            return new Techreport();
          case 'unpublished':
            return new Unpublished();
          default:
           // throw new SLException("Publication of type `$pub_type` doesn't exist.");
        }

        return null;
    }

    /**
     * When implemented in successors, should validate input form data from the user
     * and return textual report if some mandatory parameters are missing or some
     * parameters are left over.
     *
     * @param array $data ('$param name' => '$value'); value must be trimmed
     * @return string|null report
     * @throws Exception if title not given
     */
    public function validate(array $data)
    {
        extract($data);

        //title must be always given
        if (empty($title)) {
            //throw new Exception('Title is missing');
            $this->report_write("Title is missing");
        }

        return $this->report;
    }

    /**
     * Writes to the report
     *
     * @param string $msg message
     */
    protected function report_write(string $msg)
    {
        if ($this->report) {
            $this->report .= ", ";
        }

        $this->report .= $msg;
    }

    /**
     * Prints char '*' if parameter attribute is mandatory or nothing if not
     *
     * @param string $attribute
     */
    public function star(string $attribute)
    {
        print($this->starmap[$attribute]);
    }

    /**
     * @param $isbn
     * @return bool
     */
    public function valid_isbn($isbn): bool
    {
        return preg_match('/^[0-9]+-[0-9]+-[0-9]+-([0-9]+|X)$/', $isbn) && (strlen($isbn) == 13) ||
            preg_match('/^[0-9]+-[0-9]+-[0-9]+-([0-9]+|X)$/', $isbn) && (strlen($isbn) == 17);
    }

    /**
     * Returns bibtex representation of a publication with given $data.
     * Optionally uses specified abbreviation as publication identifier, if not specified uses default one.
     *
     * @param array $data
     * @param string|null $abbreviation
     * @return string bibtex data
     */
    public function get_bibtex(array $data, ?string $abbreviation = null): string {
        $data = $this->convert($data);
        $pub_type = $data['pub_type'];
        $abb = (!is_null($abbreviation) && $abbreviation !== '') ? $abbreviation : $this->abb($data);

        $bib = "@{$pub_type}{";
        $bib .= $abb !== '' ? "{$abb},\n" : "\n";

        foreach($this->bibtex_fields as $value) {
            if (isset($data[$value]) && $data[$value]) {
                if ($value === 'month') {
                    //convert numeric month to 3 letter month name
                    $date_month = Nette\DateTime::createFromFormat('!m', $data[$value]);
                    $month = $date_month->format('F');
                    $month = substr($month, 0, 3);
                    $month = strtolower($month);
                    $month = ucfirst($month);
                    $bib .= "{$this->pad}{$value} = {$month},\n";
                } else {
                    $output_value = $data[$value];

                    //wrap in quotes if contains spaces
                    if (strpos($output_value, ' ') !== false) {
                        $output_value = '"' . $output_value . '"';
                    }

                    //in title make multiple uppercase letters acronym (wrap them in curly braces)
                    if ($value === 'title') {
                        $output_value = preg_replace('/[A-Z]{2,}/', '{$0}', $output_value);
                    }

                    $bib .= "{$this->pad}{$value} = {$output_value},\n";
                }
            }
        }

        //trim comma at the end if necessary
        if ($bib[strlen($bib) - 2] == ',') {
            $bib = substr($bib, 0, strlen($bib) - 2);
        }

        $bib .= "\n}";

        return nl2br($bib);
    }

    /**
     * Do some conversions of input data to preserve right format of bibtex fields
     *
     * @param array $data
     * @return array
     */
    protected function convert(array $data): array
    {
        $pages = $data['pages'];

        if (strpos($pages, '-') !== false) {
            $pages = str_replace('-', '--', $pages);
            $data['pages'] = $pages;
        }

        return $data;
    }

    /**
     * Return abbreviation of citation
     *
     * @param array $data BibTeX fields
     * @return string
     */
    protected function abb(array $data): string
    {
        return $data['year'] ?? '';
    }
}



/**
 * Article - represents bibtex type of publication
 */
class Article extends Bibtex {

    function __construct() {
        $this->starmap = array(
            'author' => '* (or fill editor)',
            'editor' => '* (or fill author)',
            'journal' => '*',
            'year' => '*'
        );

        $this->bibtex_fields = array(
            'author', 'journal', 'title', 'year', 'month', 'volume', 'note', 'number', 'pages'
        );
    }

    /**
     * Check if all mandatory BibTeX fields are given
     *
     * @param array $data
     * @return string|null validation report
     * @throws Exception
     */
    public function validate(array $data)
    {
        parent::validate($data);

        extract($data);

        if (count($author) == 0) {
            $this->report_write('Author is missing');
        }

        if (!isset($journal) || !$journal) {
            $this->report_write('Journal is missing');
        }

        if (empty($year)) {
            $this->report_write('Year is missing');
        }

        return $this->report;
    }
}



/**
 * Book - represents bibtex type of publication
 */
class Book extends Bibtex {

    function __construct() {
        $this->starmap = array(
            'author' => '* (or fill editor)',
            'editor' => '* (or fill author)',
            'publisher' => '*',
            'year' => '*'
        );

        $this->bibtex_fields = array(
            'author', 'editor', 'publisher', 'title', 'year', 'address', 'edition', 'month', 'note', 'number', 'series', 'volume'
        );
    }

    /**
     * Check if all mandatory BibTeX fields are given
     *
     * @param array $data
     * @return string|null validation report
     * @throws Exception
     */
    public function validate(array $data)
    {
        parent::validate($data);

        extract($data);

        if (isset($author) && count($author) == 0 && empty($editor)) {
            $this->report_write('Both author and editor are missing');
        }

        if (empty($year)) {
            $this->report_write('Year is missing');
        }

        if (empty($publisher)) {
            $this->report_write('Publisher is missing');
        }

        if (isset($isbn) && $isbn) {
            $this->report_write('ISBN is missing');
        } else {
            if (!$this->valid_isbn($isbn)) {
                $this->report_write('ISBN format is wrong');
            }
        }

        return $this->report;
    }
}



/**
 * Booklet - represents bibtex type of publication
 */
class Booklet extends Bibtex {

    public function __construct()
    {
        $this->bibtex_fields = array(
            'title', 'address', 'author', 'howpublished', 'month', 'note', 'year'
        );
    }

    /**
     * Check if all mandatory BibTeX fields are given
     *
     * @param array $data
     * @return string validation report
     * @throws Exception
     */
    public function validate(array $data)
    {
        return parent::validate($data);
    }
}



/**
 * Inbook - represents bibtex type of publication
 */
class Inbook extends Bibtex {

    function __construct() {
        $this->starmap = array(
            'author' => '* (or fill editor)',
            'editor' => '* (or fill author)',
            'chapter' => '* (or fill pages range)',
            'pages' => '* (or fill chapter)',
            'publisher' => '*',
            'year' => '*'
        );

        $this->bibtex_fields = array(
            'author', 'editor', 'chapter', 'pages', 'publisher', 'title', 'year', 'address', 'edition', 'month', 'note', 'number', 'series', 'volume'
        );
    }

    /**
     * Check if all mandatory BibTeX fields are given
     *
     * @param array $data
     * @return string validation report
     * @throws Exception
     */
    public function validate(array $data)
    {
        parent::validate($data);

        extract($data);

        if (isset($author) && count($author) == 0 && empty($editor)) {
            $this->report_write('Author is missing ani není uveden editor');
        }

        if (empty($chapter) && empty($pages)) {
            $this->report_write('Both chapter and pages are empty');
        }

        if (empty($publisher)) {
            $this->report_write('Publisher is missing');
        }

        if (empty($year)) {
            $this->report_write('Year is missing');
        }

        if (isset($isbn) && $isbn) {
            $this->report_write('ISBN is missing');
        } else {
            if (!$this->valid_isbn($isbn)) {
                $this->report_write('ISBN format is wrong');
            }
        }

        return $this->report;
    }
}



/**
 * Incollection - represents bibtex type of publication
 */
class Incollection extends Bibtex {

    function __construct() {
        $this->starmap = array(
            'author' => '*',
            'booktitle' => '*',
            'publisher' => '*',
            'year' => '*'
        );

        $this->bibtex_fields = array(
            'author', 'booktitle', 'publisher', 'title', 'year', 'address', 'chapter', 'edition', 'editor', 'month', 'note', 'number', 'pages', 'series', 'volume'
        );
    }

    /**
     * Check if all mandatory BibTeX fields are given
     *
     * @param array $data
     * @return string validation report
     * @throws Exception
     */
    public function validate(array $data)
    {
        parent::validate($data);

        extract($data);

        if (isset($author) && count($author) == 0) {
            $this->report_write('Author is missing');
        }

        if (empty($publisher)) {
            $this->report_write('Publisher is missing');
        }

        if (empty($booktitle)) {
            $this->report_write('Booktitle is missing');
        }

        if (empty($year)) {
            $this->report_write('Year is missing');
        }

        if (isset($isbn) && $isbn) {
            $this->report_write('ISBN is missing');
        } else {
            if (!$this->valid_isbn($isbn)) {
                $this->report_write('ISBN format is wrong');
            }
        }

        return $this->report;
    }
}



/**
 * Inproceedings - represents bibtex type of publication
 */
class Inproceedings extends Bibtex {

    function __construct() {
        $this->starmap = array(
            'author' => '*',
            'booktitle' => '*',
            'year' => '*'
        );

        $this->bibtex_fields = array(
            'author', 'booktitle', 'title', 'year', 'address', 'editor', 'location', 'month', 'note', 'number', 'organization', 'pages', 'publisher', 'series', 'volume'
        );
    }

    /**
     * Check if all mandatory BibTeX fields are given
     *
     * @param array $data
     * @return string validation report
     * @throws Exception
     */
    public function validate(array $data)
    {
        parent::validate($data);

        extract($data);

        if (count($author) == 0) {
            $this->report_write('Author is missing');
        }

        //if (empty($conference) && empty($booktitle))
        //	$this->report_write('No conference was chosen');

        if (empty($conference2) && empty($booktitle)) {
            $this->report_write('No conference was chosen');
        }

        if (empty($conference_year) && empty($booktitle)) {
            $this->report_write('No conference year was chosen');
        }

        if (empty($year)) {
            $this->report_write('Year is missing');
        }

        return $this->report;
    }
}



/**
 * Manual - represents bibtex type of publication
 */
class Manual extends Bibtex {

    public function __construct()
    {
        $this->bibtex_fields = array(
            'title', 'address', 'author', 'edition', 'month', 'note', 'organization', 'year'
        );
    }

    /**
     * Check if all mandatory BibTeX fields are given
     *
     * @param array $data
     * @return string validation report
     * @throws Exception
     */
    public function validate(array $data)
    {
        return parent::validate($data);
    }
}



/**
 * Mastersthesis - represents bibtex type of publication
 */
class Mastersthesis extends Bibtex {

    function __construct() {
        $this->starmap = array(
            'author' => '*',
            'school' => '*',
            'year' => '*'
        );

        $this->bibtex_fields = array(
            'author', 'school', 'title', 'year', 'address', 'month', 'note'
        );
    }

    /**
     * Check if all mandatory BibTeX fields are given
     *
     * @param array $data
     * @return string validation report
     * @throws Exception
     */
    public function validate(array $data)
    {
        parent::validate($data);

        extract($data);

        if (count($author) == 0)
          $this->report_write('Author is missing');

        if (empty($school))
          $this->report_write('School is missing');

        if (empty($year))
          $this->report_write('Year is missing');

        return $this->report;
    }
}



/**
 * Misc - represents bibtex type of publication
 */
class Misc extends Bibtex {

    public function __construct()
    {
        $this->bibtex_fields = array(
            'title', 'author', 'editor', 'howpublished', 'month', 'note', 'year'
        );
    }

    /**
     * Check if all mandatory BibTeX fields are given
     *
     * @param array $data
     * @return string validation report
     * @throws Exception
     */
    public function validate(array $data)
    {
        return parent::validate($data);
    }
}



/**
 * Phdthesis - represents bibtex type of publication
 */
class Phdthesis extends Bibtex {

    function __construct() {
        $this->starmap = array(
            'author' => '*',
            'school' => '*',
            'year' => '*'
        );

        $this->bibtex_fields = array(
            'author', 'school', 'title', 'year', 'address', 'month', 'note'
        );
    }

    /**
     * Check if all mandatory BibTeX fields are given
     *
     * @param array $data
     * @return string validation report
     * @throws Exception
     */
    public function validate(array $data)
    {
        parent::validate($data);

        extract($data);

        if (count($author) == 0) {
            $this->report_write('Author is missing');
        }

        if (empty($school)) {
            $this->report_write('School is missing');
        }

        if (empty($year)) {
            $this->report_write('Year is missing');
        }

        return $this->report;
    }
}



/**
 * Proceedings - represents bibtex type of publication
 */
class Proceedings extends Bibtex {

    function __construct() {
        $this->starmap = array(
            'year' => '*'
        );

        $this->bibtex_fields = array(
            'title', 'year', 'editor', 'location', 'month', 'note', 'number', 'organization', 'publisher', 'series', 'volume'
        );
    }

    /**
     * Check if all mandatory BibTeX fields are given
     *
     * @param array $data
     * @return string validation report
     * @throws Exception
     */
    public function validate(array $data)
    {
        parent::validate($data);

        extract($data);

        if (empty($year)) {
            $this->report_write('Year is missing');
        }

        return $this->report;
    }
}



/**
 * Techreport - represents bibtex type of publication
 */
class Techreport extends Bibtex {

    function __construct() {
        $this->starmap = array(
            'author' => '*',
            'institution' => '*',
            'year' => '*'
        );

        $this->bibtex_fields = array(
            'author', 'institution', 'title', 'year', 'address', 'month', 'note', 'number', 'type'
        );
    }

    /**
     * Check if all mandatory BibTeX fields are given
     *
     * @param array $data
     * @return string validation report
     * @throws Exception
     */
    public function validate(array $data)
    {
        parent::validate($data);

        extract($data);

        if (count($author) == 0) {
            $this->report_write('Author is missing');
        }

        if (empty($institution)) {
            $this->report_write('Institution is missing');
        }

        if (empty($year)) {
            $this->report_write('Year is missing');
        }

        return $this->report;
    }
}



/**
 * Unpublished - represents bibtex type of publication
 */
class Unpublished extends Bibtex {

    function __construct() {
        $this->starmap = array(
            'author' => '*',
            'note' => '*'
        );

        $this->bibtex_fields = array(
            'author', 'note', 'title', 'month', 'year'
        );
    }

    /**
     * Check if all mandatory BibTeX fields are given
     *
     * @param array $data
     * @return string validation report
     * @throws Exception
     */
    public function validate(array $data)
    {
        parent::validate($data);

        extract($data);

        if (count($author)) {
            $this->report_write('Author is missing');
        }

        if (empty($note)) {
            $this->report_write('Note is missing');
        }

        if (empty($year)) {
            $this->report_write('Year is missing');
        }

        return $this->report;
    }
}
