<?php

namespace App\Model;


use Nette\Database\Context;
use Nette\Database\ResultSet;
use Nette\Database\Table\ActiveRow;
use Nette\Database\Table\Selection;

class PublicationGroup extends Base {

    /**
     * Name of the database table
     * @var string
     */
    protected $tableName = 'publication_group';

    /** @var Author */
    protected $authorModel;

    public function __construct(Author $authorModel, Context $db)
    {
        parent::__construct($db);
        $this->authorModel = $authorModel;
    }

    /**
     * @param int $id
     * @return Selection
     */
    public function getAllPublications(int $id): Selection
    {
        return $this->database->table('publication_group_has_publication')
            ->select('publication.journal.name AS journal,
                    publication.journal.id AS journal_id,
                    publication.publisher.name AS publisher,
                    publication.conference_year.location AS location, 
                    publication.conference_year.name AS name,
                    publication.conference_year.id AS cy_id,
                    publication.type_of_report AS type, 
                    publication.id, 
                    publication.pub_type, 
                    publication.title, 
                    publication.volume, 
                    publication.number, 
                    publication.pages, 
                    publication.issue_month AS month_eng, 
                    publication.issue_year AS year, 
                    publication.url, 
                    publication.note, 
                    publication.editor, 
                    publication.edition, 
                    publication.address, 
                    publication.howpublished, 
                    publication.chapter, 
                    publication.booktitle, 
                    publication.school,
                    publication.institution, 
                    publication.conference_year_id')
            ->where('publication_group_has_publication.publication_group_id = ?', $id);
    }

    /**
     * @param int $publication_id
     * @param int $user_id
     * @return ResultSet
     */
    public function getPublicationsUsersGroups(int $publication_id, int $user_id): ResultSet
    {
        return $this->database->query('
            SELECT pg.id FROM publication_group pg
            JOIN publication_group_has_publication pghp on pg.id = pghp.publication_group_id
            JOIN publication p on p.id = pghp.publication_id
            JOIN submitter_is_in_publication_group siipg on pg.id = siipg.publication_group_id
            WHERE siipg.submitter_id = ? AND p.id = ?
        ', $user_id, $publication_id);
    }

    /**
     * @param int $id
     * @return ResultSet
     */
    public function getAllMembers(int $id): ResultSet
    {
        return $this->database->query('
            SELECT s.id, s.name, s.surname, s2.name AS added_by_name, s2.surname AS added_by_surname, spg.added_on, spg.deleted_on FROM submitter_is_in_publication_group spg 
            JOIN submitter s ON spg.submitter_id = s.id
            JOIN submitter s2 ON spg.added_by_submitter_id = s2.id
            WHERE publication_group_id = ?
        ', $id);
    }

    /**
     * @param string $name
     * @return false|ActiveRow
     */
    public function findOneByName(string $name)
    {
        return $this->findOneBy(['name' => $name]);
    }

    /**
     * @param int $publicationGroupId
     */
    public function deleteAssociatedRecords(int $publicationGroupId): void
    {
        $this->database->table('publication_group_has_publication')
            ->where(['publication_group_id' => $publicationGroupId])
            ->delete();

        $this->database->table('submitter_is_in_publication_group')
            ->where(['publication_group_id' => $publicationGroupId])
            ->delete();

        $this->find($publicationGroupId)->delete();
    }

    /**
     * Sets publication group members to specified array, deletes and inserts as necessary.
     * @param int $group_id
     * @param array $submitter_ids
     * @param int $current_user_id
     * @return void
     */
    public function setGroupSubmitters(int $group_id, array $submitter_ids, int $current_user_id): void
    {
        $current_submitters = $this->database->table('submitter_is_in_publication_group')
            ->select('submitter_id')
            ->where('publication_group_id = ?', $group_id)
            ->fetchPairs(null, 'submitter_id');

        $to_be_inserted = array_diff($submitter_ids, $current_submitters);
        $to_be_removed = array_diff($current_submitters, $submitter_ids);

        $insert_array = [];

        foreach ($to_be_inserted as $item) {
            $insert_array[] = [
                'publication_group_id' => $group_id,
                'submitter_id' => $item,
                'added_by_submitter_id' => $current_user_id
            ];
        }

        $this->database->table('submitter_is_in_publication_group')
            ->where('publication_group_id = ?', $group_id)
            ->where('submitter_id IN (?)', $to_be_removed)
            ->delete();

        if (count($insert_array)) {
            $this->database->table('submitter_is_in_publication_group')
                ->insert($insert_array);
        }
    }

    /**
     * @param int $group_id
     * @return Selection
     */
    public function getGroupSubmitters(int $group_id): Selection
    {
        return $this->database->table('submitter_is_in_publication_group')
            ->select('submitter_id')
            ->where('publication_group_id = ?', $group_id);
    }

    /**
     * @return ResultSet
     */
    public function getEligibleSubmitters(): ResultSet
    {
        return $this->database->query('
            SELECT DISTINCT s.* FROM submitter s
            JOIN user_role ur ON s.id = ur.user_id
            WHERE ur.role = \'submitter\' OR ur.role = \'conference-moderator\' OR ur.role = \'admin\'
        ');
    }

    /**
     * @param int $group_id
     * @return ResultSet
     */
    public function getEligibleSubmittersWhoArentInGroup(int $group_id): ResultSet
    {
        return $this->database->query('
            SELECT DISTINCT s.* FROM submitter s
            JOIN user_role ur ON s.id = ur.user_id
            WHERE (ur.role = \'submitter\' OR ur.role = \'conference-moderator\' OR ur.role = \'admin\')
                AND s.id NOT IN (
                    SELECT submitter_id 
                    FROM submitter_is_in_publication_group 
                    WHERE submitter_is_in_publication_group.publication_group_id = ?
                )
        ', $group_id);
    }

    /**
     * @param int $submitter_id
     * @return ResultSet
     */
    public function getSubmittersGroups(int $submitter_id): ResultSet
    {
        return $this->database->query('
            SELECT pg.* FROM publication_group pg 
            JOIN submitter_is_in_publication_group spg ON spg.publication_group_id = pg.id
            WHERE spg.submitter_id = ?
        ', $submitter_id);
    }

    /**
     * @param int $pub_id
     * @param int $pubgroup_id
     * @param int $submitter_id
     * @return void
     */
    public function addPublicationToPubGroup(int $pub_id, int $pubgroup_id, int $submitter_id): void
    {
        $count = $this->database->table('publication_group_has_publication')
            ->select('id')
            ->where(['publication_group_id' => $pubgroup_id, 'publication_id' => $pub_id])
            ->count();

        if ($count === 0) {
            $this->database->table('publication_group_has_publication')
                ->insert([
                    'publication_group_id' => $pubgroup_id,
                    'publication_id' => $pub_id,
                    'added_by_submitter_id' => $submitter_id
                ]);
        }
    }

    /**
     * @param int $pub_id
     * @param int $pubgroup_id
     * @return void
     */
    public function deletePublicationFromPubGroup(int $pub_id, int $pubgroup_id): void
    {
        $this->database->table('publication_group_has_publication')
            ->where(['publication_group_id' => $pubgroup_id, 'publication_id' => $pub_id])
            ->delete();
    }

    /**
     * @param int $group_id
     * @return array
     */
    public function getPublicationsForAdd(int $group_id): array
    {
        $arr = [];

        $pubIdsInGroup = $this->database->table('publication_group_has_publication')
            ->select('publication_id')
            ->where('publication_group_id', $group_id)
            ->fetchPairs(null, 'publication_id');

        $query = $this->database->table('publication')
            ->where('id NOT IN (?)', $pubIdsInGroup);

        $publications = $query->order('title');

        $publications = $publications->fetchAssoc('id');
        $publicationsIds = array_keys($publications);
        $authors = $this->authorModel->getAuthorsByMultiplePubIds($publicationsIds);

        foreach ($publications as $id => $pub) {
            $pubAuthors = $authors[$id] ?? [];
            $pubAuthorsStrings = [];

            foreach ($pubAuthors as $author) {
                $pubAuthorsStrings[] = $author['name'] . ' ' . ($author['middlename'] ? $author['middlename'] . ' ' : '') . ($author['surname']);
            }

            $arr[$id] = $pub['title'] . ' (' . join('; ', $pubAuthorsStrings) . '; id: ' . $id . ')';
        }

        return $arr;
    }
}
