<?php

namespace App\Presenters;

use App\Components\Publication\PublicationControl;
use App\CrudComponents\Journal\PublicationGroupCrud;
use App\Forms\BaseForm;
use App\Helpers\Bibtex;
use App\Model;
use Nette\Application\AbortException;
use Nette\Application\UI\Form;


class PublicationGroupPresenter extends SecuredPresenter {

    /** @var array */
    private $publicationIds = [];

    /** @var int */
    private $groupId = 0;

    /** @var Model\PublicationGroup @inject */
    public $publicationGroupModel;

    /** @var Model\Author @inject */
    public $authorModel;

    /** @var Model\Submitter @inject */
    public $submitterModel;


    /**
     * @return PublicationControl
     */
    protected function createComponentPublication(): PublicationControl
    {
        return new PublicationControl();
    }

    /**
     * @return Form
     */
    protected function createComponentSelectPublicationsForm(): Form
    {
        $form = new Form();

        foreach ($this->publicationIds as $id) {
            $form->addCheckbox($id)->setHtmlAttribute('class', 'select-publications-checkbox');
        }

        $form->addSubmit('submit');

        $form->onSuccess[] = function(Form $form, array $values) {
            $ids = [];

            foreach ($values as $key => $value) {
                if ($value) {
                    $ids[] = $key;
                }
            }

            if (count($ids) === 0) {
                $this->presenter->redirect('detail', $this->groupId);
            }

            $ids_string = join(',', $ids);
            $this->presenter->redirect('export', ['id' => $this->groupId, 'publications' => $ids_string]);
        };

        return $form;
    }

    /**
     * @return PublicationGroupCrud
     */
    public function createComponentCrud(): PublicationGroupCrud
    {
        $c = new PublicationGroupCrud(
            $this->user,
            $this->publicationGroupModel,
            $this->submitterModel,
            $this, 'crud'
        );

        $c->onAdd[] = function ($row) {
            $this->successFlashMessage(sprintf('Publication group %s has been created successfully', $row->name));

            $this->redirect(':PublicationGroup:detail', $row->id);
        };

        $c->onDelete[] = function ($row) {
            $this->successFlashMessage(sprintf('Publication group %s has been deleted successfully', $row->name));

            $this->redirect(':PublicationGroup:default');
        };

        $c->onEdit[] = function ($row) {
            $this->successFlashMessage(sprintf('Publication group %s has been edited successfully', $row->name));

            $this->redirect(':PublicationGroup:detail', $row->id);
        };

        return $c;
    }

    /**
     * @param string $name
     * @return BaseForm
     */
    public function createComponentAddPublicationToPublicationGroupForm(string $name): BaseForm
    {
        $form = new BaseForm();

        $form->addSelect('publication', 'Publication', $this->publicationGroupModel->getPublicationsForAdd($this->groupId));

        $form->addSubmit('save', 'Save');

        $form->onError[] = function () {
            $this->redrawControl('addPublicationToPublicationGroupForm');
        };

        $form->onSuccess[] = function (BaseForm $form) {
            $formValues = $form->getValuesTransformed();

            $this->publicationGroupModel->addPublicationToPubGroup($formValues['publication'], $this->groupId, $this->user->getId());

            if ($this->presenter->isAjax()) {
                $form->clearValues();
                $this->template->publicationAdded = true;
                $this->redrawPublications();
            } else {
                $this->redirect('this');
            }
        };

        return $form;
    }

    /**
     * @return void
     */
	public function actionDefault(): void
    {
        $this->template->groups = $this->publicationGroupModel->getSubmittersGroups($this->user->id)->fetchAll();
	}

    /**
     * @param int $id
     * @return void
     */
    public function actionDetail(int $id): void
    {
        $group = $this->publicationGroupModel->find($id);
        $members = $this->publicationGroupModel->getAllMembers($id)->fetchAll();
        $publications = $this->publicationGroupModel->getAllPublications($id)->fetchAll();

        $publicationIds = [];
        foreach ($publications as $publication) {
            $publicationIds[] = $publication->id;
        }
        $this->publicationIds = $publicationIds;
        $this->groupId = $id;
        $authors = $this->authorModel->getAuthorsByMultiplePubIds($publicationIds);

        $this->template->group = $group;
        $this->template->members = $members;
        $this->template->publications = $publications;
        $this->template->authors = $authors;
    }

    /**
     * @param int $id
     * @param string $publications
     * @return void
     */
    public function actionExport(int $id, string $publications): void
    {
        $group = $this->publicationGroupModel->find($id);
        $publications_ids = explode(',', $publications);
        $publications = $this->publicationModel->getMultiplePubInfoByIds($publications_ids)->fetchAssoc('id');
        $bibtex_exports = [];

        foreach ($publications_ids as $key => $id) {
            $bibtex = Bibtex::create($publications[$id]['pub_type']);
            $bibtex_exports[$id] = $bibtex->get_bibtex($publications[$id], ($key + 1));
        }

        $this->template->group = $group;
        $this->template->ids = $publications_ids;
        $this->template->bibtex_exports = $bibtex_exports;
        $this->template->publications = $publications;
    }

    /**
     * @param int $pub_id
     * @param int $pubgroup_id
     * @return void
     * @throws AbortException
     */
    public function handleDeletePublicationFromPubGroup(int $pub_id, int $pubgroup_id): void
    {
        if ($this->user->isInRole('admin') || $this->user->isInRole('submitter')) {
            $this->publicationGroupModel->deletePublicationFromPubGroup($pub_id, $pubgroup_id);

            $this->flashMessage('Publication has been deleted from the publication group successfully.', 'alert-success');

            if ($this->isAjax()) {
                $this->redrawPublications();
            } else {
                $this->redirect('this');
            }
        }
    }

    private function redrawPublications(): void
    {
        $publications = $this->publicationGroupModel->getAllPublications($this->groupId)->fetchAll();

        $publicationIds = [];
        foreach ($publications as $publication) {
            $publicationIds[] = $publication->id;
        }
        $this->publicationIds = $publicationIds;
        $authors = $this->authorModel->getAuthorsByMultiplePubIds($publicationIds);

        $this->template->publications = $publications;
        $this->template->authors = $authors;

        $this->redrawControl('pubgroupPublications');
        $this->redrawControl('addPublicationToPublicationGroupForm');
        $this->redrawControl('flashMessages');
    }

}
